#!/bin/bash

set -ex

keychain -q
. ~/.keychain/$(hostname)-sh
export KEY_FILE=/home/max/.ssh/id_rsa.pub
export PUBLIC_KEY=$(cat $KEY_FILE)

export WORKSPACE=/home/max/other-workspace.d
cd $WORKSPACE
#rm -rf $WORKSPACE/*
export GIT_DIR=$WORKSPACE/ubuntu-core-default-user-image-scripts
#git clone https://github.com/om26er/ubuntu-core-default-user-image-scripts.git $GIT_DIR
cd $GIT_DIR

export KEY_FILE=/home/max/.ssh/id_rsa.pub
export PUBLIC_KEY=$(cat $KEY_FILE)
sudo PUBLIC_KEY="$PUBLIC_KEY" ./create-image.sh

IMAGE=$GIT_DIR/ubuntu-core-16-pc-amd64.img
NAME="$(basename $IMAGE)"
NAME="${NAME%.*}"

LOCKDIR="/var/lock/upgrade-testing"
if ! mkdir -p $LOCKDIR; then
    echo "Failed to create lock directory $LOCKDIR . Aborting!"
    exit 1
fi

log_msg() {
    printf "$(date '+%F %X'): $@"
}

log_info_msg() {
    log_msg "Info: $@\n"
}

log_failure_msg() {
    log_msg "Failure: $@\n"
}

get_free_port() {
    srcport=54322
    lockfilebase="$LOCKDIR/ssh"

    for port in $(seq 0 100); do
        srcport=$((srcport + 1))
        srcaddr="(0.0.0.0|127.0.0.1):$srcport"

        lockfile=${lockfilebase}.${srcport}.lock

        log_info_msg "Trying port $srcport"
        # keep open lock fd forever (i. e. until end of program)
        fd=8
        exec 8>>$lockfile
        if flock -xn $fd; then
            if [ -z "$( netstat -t -l -n |grep -E $srcaddr)" ]; then
                log_info_msg "Successfully locked port $srcport"
                export PORT=$srcport
                break
            else
                flock -u -n $fd
                rm -f $lockfile
            fi
        fi
        log_info_msg "failed to lock port $srcport"
        sleep 1
    done

    if [ $port -eq 100 ]; then
        log_failure_msg "Failed to find free port. Aborting!"
        exit 1
    fi

    return $srcport
}

run_cmd() {
    ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no $USER@localhost -p $PORT "$@"
}

setup_key() {
    for x in $(seq 1 60) ; do
        echo "SSH key installation attempt $x">&2
        if (sshpass -p $PASSWORD scp -P $PORT $KEY_FILE -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no $USER@localhost:.ssh/authorized_keys) ; then
            return
        fi
        sleep 10
    done
}

wait_for_ssh() {
    for x in $(seq 1 60) ; do
        echo "SSH connection attempt $x">&2
        if (run_cmd "/bin/true") ; then
            return
        fi
        sleep 10
    done
}

cleanup() {
    if [ -n "$PID" ] ; then
        kill $PID
    fi
}

reboot() {
    run_cmd "sudo shutdown -r" # Exits nonzero if I just use reboot
    sleep 120
    wait_for_ssh
}

setup_proxy(){
    cat >$WORKSPACE/proxy.conf <<EOF
[Service]
Environment=https_proxy=$job_https_proxy
EOF
    scp -P $PORT $WORKSPACE/proxy.conf $USER@localhost:/tmp/proxy.conf
    run_cmd "sudo mkdir -p /etc/systemd/system/snapd.service.d"
    run_cmd "sudo cp /tmp/proxy.conf /etc/systemd/system/snapd.service.d/proxy.conf"
    run_cmd "sudo systemctl daemon-reload"
    run_cmd "sudo systemctl restart snapd.service"
    cat >$WORKSPACE/environment <<EOF
HTTP_PROXY=$job_http_proxy
HTTPS_PROXY=$job_https_proxy
EOF
    scp -P $PORT $WORKSPACE/environment $USER@localhost:/tmp/environment
    run_cmd "sudo snap install --beta --devmode classic"
    run_cmd "cat /tmp/environment | sudo classic sudo tee -a /etc/environment"
}

trap cleanup SIGHUP SIGINT SIGTERM

IMAGE=${IMAGE-$1}

if [ -z "$IMAGE" ] ; then
    echo "Image required as first argument">&2
    exit 2
fi

export USER=test
export PORT
get_free_port || true

kvm -name $NAME -m 2048 -smp 2 -cpu host -nographic -drive if=virtio,file=$IMAGE -net nic,model=virtio -net user,hostfwd=tcp::$PORT-:22 -snapshot &
export PID=$!
sleep 60 # TODO: maybe do ssh with explicit nopassword?

#setup_key
wait_for_ssh

if [ -n "$job_https_proxy" ] ; then
    setup_proxy
fi

#run_cmd "sudo snap refresh --candidate core"
#reboot
#run_cmd "sudo snap refresh --candidate pc"
#run_cmd "sudo snap refresh --candidate pc-kernel"
#reboot

#export GIT_URL=${GIT_URL-https://github.com/snapcore/snapd/}
export GIT_DIR=$WORKSPACE/snapd
#rm -rf $GIT_DIR
#git clone $GIT_URL $GIT_DIR
cd $GIT_DIR
export SPREAD_EXTERNAL_ADDRESS=127.0.0.1:$PORT
./tests/lib/external/prepare-ssh.sh 127.0.0.1 $PORT $USER
git init || true # TODO: try to get rid of this
#sed "s@go get@HTTPS_PROXY=$https_proxy go get@" -i spread.yaml
#run_cmd "sudo classic cat /etc/environment"
spread -v -reuse external:ubuntu-core-16-64 | tee $WORKSPACE/spread-output

cleanup

$HOME/resources/spread.py $WORKSPACE/spread-output > $WORKSPACE/junit.xml # TODO: land this somewhere
