#!/usr/bin/python3
# -*- coding: utf8 -*-
## Parse the test results from spread tool
import re
import sys
from dateutil.parser import parse

re_executing = re.compile('(^.*) Executing external\:(.*)\.\.\.')
re_error = re.compile('^.* Error executing external\:(.*) \:\s*(.*)')
re_error_output = re.compile('^-----$')
re_restoring = re.compile('(^.*) Restoring external\:(.*)\.\.\.')

def test_re_executing():
    """ Test preparing regex """
    string = '2016/10/27 16:59:54 Executing external:ubuntu-core-16-arm-64:tests/main/revert:fake...'
    f = re_executing.findall(string)
    result = len(f) == 1 and f[0][0] == '2016/10/27 16:59:54' and f[0][1] == 'ubuntu-core-16-arm-64:tests/main/revert:fake'
    return result

def test_re_error():
    """ Test executing regex """
    string = '2016/10/27 17:39:45 Error executing external:ubuntu-core-16-arm-64:tests/main/ubuntu-core-device-reg :'
    f = re_error.findall(string)
    string_eof = '2016/11/03 14:12:22 Error executing external:ubuntu-core-16-arm-64:tests/main/ubuntu-core-reboot : kill-timeout reached, cannot reconnect to external:ubuntu-core-16-arm-64 after reboot: dial tcp 192.168.1.38:22: getsockopt: no route to host'
    f_eof = re_error.findall(string_eof)
    result = len(f) == 1 and f[0][0] == 'ubuntu-core-16-arm-64:tests/main/ubuntu-core-device-reg' and f[0][1] == '' and len(f_eof) == 1 and f_eof[0][0] == 'ubuntu-core-16-arm-64:tests/main/ubuntu-core-reboot' and f_eof[0][1] == 'kill-timeout reached, cannot reconnect to external:ubuntu-core-16-arm-64 after reboot: dial tcp 192.168.1.38:22: getsockopt: no route to host'
    return result

def test_re_error_output():
    """ Test error output regex """
    string = '-----'
    f = re_error_output.findall(string)
    result = len(f) == 1
    return result

def test_re_restoring():
    """ Test restoring regex """
    string = '2016/10/27 17:06:10 Restoring external:ubuntu-core-16-arm-64:tests/main/ubuntu-core-reboot...'
    f = re_restoring.findall(string)
    result = len(f) == 1 and f[0][0] == '2016/10/27 17:06:10' and f[0][1] == 'ubuntu-core-16-arm-64:tests/main/ubuntu-core-reboot'
    return result

def parse_execute(filename):
    """ Return the tests executed """
    spread_text = open(filename)
    line = spread_text.readline()
    tests = []
    test_length = {}
    errors = {}
    error = 0
    error_name = ""
    error_output = ""
    test_name = ""
    test_datetime = ""
    while (line):
        if (error > 0):
            # Capture error output blocks
            # This comes in multiple lines
            f = re_error_output.findall(line)
            if (len(f) > 0):
                error += 1
            elif (error > 2):
                errors[error_name] = error_output
                error_name = ""
                error_output = ""
                error = 0
            else:
                error_output += line
        # Find "Executing..."
        f = re_executing.findall(line)
        if (len(f) > 0):
            # Save test start datetime
            (test_datetime, test_name) = f[0]
        # Find "Error executing..."
        f = re_error.findall(line)
        if (len(f) > 0):
            error_name = f[0][0]
            error_output = f[0][1]
            if (error_output == ''):
                # Multiline error
                error = 1
            else:
                errors[error_name] = error_output
        # Find "Restoring..."
        f = re_restoring.findall(line)
        if (len(f) > 0):
            # Calculate delta time from Executing to Restoring
            (datetime, name) = f[0]
            dt_end = parse(datetime)
            dt_begin = parse(test_datetime)
            delta = dt_end - dt_begin
            # Append test to the lists
            tests.append(test_name)
            test_length[test_name] = delta.total_seconds()
        # Read next line
        line = spread_text.readline()
    # Close file
    spread_text.close()
    return tests, errors, test_length

def xunify(test_list, error_list, test_length):
    """ Return the xunit xml """
    xml = """<?xml version="1.0" encoding="UTF-8"?>\n"""
    xml += """<testsuite name="spread.yaml" tests="{}" errors="0" failures="{}" skip="0">\n""".format(len(test_list), len(error_list))
    for test in test_list:
        if (test in error_list):
            xml += """  <testcase name="{}" time="{}">\n""".format(test, test_length[test])
            xml += """    <failure>{}</failure>\n""".format(error_list[test])
            xml += """  </testcase>\n"""
        else:
            xml += """  <testcase name="{}" time="{}"></testcase>\n""".format(test, test_length[test])
    xml += """</testsuite>\n"""
    return xml

if (__name__ == '__main__'):
    # print(test_re_executing())
    # print(test_re_error())
    # print(test_re_error_output())
    # print(test_re_restoring())
    for filename in sys.argv[1:]:
        (tests, errors, length) = parse_execute(filename)
        print(xunify(tests, errors, length))
