#!/usr/bin/python3
#
# Copyright 2017, Canonical (http://www.canonical.com/)
#
# Authors:
#    Víctor R. Ruiz <victor.ruiz@canonical.com>
#
# This software is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software. If not, see <http://www.gnu.org/licenses/>.
#

import pexpect
import argparse
import re
import serial
import sys
import time

from pexpect import fdpexpect
from serial import SerialException
from time import sleep

# Get arguments
parser = argparse.ArgumentParser()
parser.add_argument('port', help='Port of the serial console to the Ubuntu Core image')
parser.add_argument('email', help='Email address of the Ubuntu One account for setup')
args = parser.parse_args()
port = args.port
email = args.email

# Setup serial connection
print('--> Serial port')
retry = 0
max_retry = 10
delay = 10
# Try to connect during (delay * max_retry) seconds
while (retry < max_retry):
    try:
        usb = serial.Serial(
            baudrate=115200,
            port=port,
        )
        usb.isOpen()
    except (serial.serialutil.SerialException, FileNotFoundError):
        print('Cannot open serial port. Retrying in {} seconds ({} of {})...'.format(delay, retry + 1, max_retry))
        time.sleep(10)
        retry += 1
    else:
        break
if (retry == max_retry):
    # Abort
    print('Couldn\'t connect to the serial device.')
    exit()

# Open serial connection and log file
log = open('core-install.log', 'a')
expect = fdpexpect.fdspawn(usb, logfile=log, encoding='utf-8')
# Start console-conf
expect.expect('Press enter to configure.', timeout=480)
print('--> Enter to configure')
expect.sendline('')

# Start configuration
expect.expect(' OK ', timeout=20)
print('--> Start')
expect.sendline('')

# Network configuration
# Get the row of the cursor, to know that it is in the Done position
print('--> Network configuration')
row_before = 0
row = 0
tries = 0
r_xy = re.compile("\x1b\[K\x1b\[([0-9;]*)H\x1b\[\?25h")
EXPECTED_ROW = 17
while (row != EXPECTED_ROW):
    expect.send('\t')
    expect.expect(pexpect.TIMEOUT, timeout=5)
    expect.send('\f')
    expect.expect(pexpect.TIMEOUT, timeout=5)
    f = r_xy.findall(expect.before)
    if (len(f) > 0):
        (r, col) = f[0].split(';')
        print('Current row: {}'.format(row))
        row = int(r)
    else:
        print('Something wrong happened')
    if (row < row_before):
        break
    row_before = row
    tries = tries + 1
    if (tries > 4):
        break
if (row != EXPECTED_ROW):
    print('Cannot find Done button')
    exit()
# Network done
expect.sendline('')

# Ubuntu One account
print('--> U1')
expect.expect('Email address', timeout=5)
expect.sendline(email)

# User created
expect.expect(pexpect.TIMEOUT, timeout=10)
address = ''
r = re.compile('ssh ([a-z0\.]+\@[0-9\.]+)')
f = r.findall(expect.before)
if (len(f) == 0):
    print('Check device configuration')
else:
    address = f[0]
print("--> {}".format(address))

# Finish setup
expect.sendline('')  # Finish button
expect.sendline('')  # Repeat to get message
expect.expect('Configuration Complete', timeout=10)
print('--> Device configured')
